//
//  ScreenBrightnessEditorView.swift
//  USB Meter
//

import SwiftUI

struct ScreenBrightnessEditorView: View {
    @EnvironmentObject private var meter: Meter

    var body: some View {
        Picker("", selection: self.$meter.screenBrightness) {
            ForEach(0...5, id: \.self) { value in
                Text("\(value)").tag(value)
            }
        }
        .pickerStyle(SegmentedPickerStyle())
    }
}