//
//  Decimal.swift
//  USB Meter
//
//  Created by Bogdan Timofte on 09/03/2020.
//  Copyright © 2020 Bogdan Timofte. All rights reserved.
//

import Foundation

import Foundation

extension Decimal {

    var doubleValue:Double {
        return NSDecimalNumber(decimal:self).doubleValue
    }

}
